/*
 Copyright (c) 2014, Pierre-Olivier Latour
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 * The name of Pierre-Olivier Latour may not be used to endorse
 or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL PIERRE-OLIVIER LATOUR BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "XLLogger.h"
#import "XLTCPClient.h"

@class XLTCPClientLogger;

/**
 *  The XLTCPClientLoggerConnection is an abstract class to implement connections
 *  for XLTCPClientLogger: it cannot be used directly.
 */
@interface XLTCPClientLoggerConnection : XLTCPClientConnection

/**
 *  Returns the XLTCPClientLogger that owns the connection.
 *
 *  @warning This returns nil after the connection has been closed.
 */
@property(nonatomic, assign, readonly) XLTCPClientLogger* logger;

@end

/**
 *  The XLTCPClientLogger class is a base class for loggers that connect
 *  to TCP servers.
 *
 *  It simply sends log records formatted as a string to the server.
 */
@interface XLTCPClientLogger : XLLogger

/**
 *  Returns the XLTCPClient used internally.
 */
@property(nonatomic, readonly) XLTCPClient* TCPClient;

/**
 *  Configures if the TCP client blocks when sending new log records to the
 *  connected server.
 *
 *  The default value is NO.
 */
@property(nonatomic) BOOL usesAsynchronousLogging;

/**
 *  Returns the class to use to instantiate client connections.
 *
 *  The default implementation returns [XLTCPClientLoggerConnection class].
 */
+ (Class)connectionClass;

/**
 *  This method is the designated initializer for the class.
 *
 *  @warning The TCP client is not running until the logger is opened.
 */
- (instancetype)initWithHost:(NSString*)hostname port:(NSUInteger)port;

@end
